/*
 * Copyright 2016 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


/**
 * @file start_s3.c
 * Implementation of the s3 DSI start session
 */
#include "globus_i_s3.h"

/**
 * @brief start
 * @details
 * This function is called when a new session is initialized, ie a user 
 * connects to the server.  This hook gives the dsi an opportunity to
 * reject the user, or set internal state that will be threaded through to all
 * other function calls associated with this session.
 */
void
globus_i_s3_start(
    globus_gfs_operation_t              op,
    globus_gfs_session_info_t          *session_info)
{
    globus_i_s3_handle_t               *s3_handle = NULL;
    char                                userline[512];
    char                               *accessKeyId = userline,
                                       *secretAccessKey = NULL,
                                       *securityToken = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusS3Enter();

    if (globus_i_s3_map_filename != NULL)
    {
        FILE                           *mapfile = NULL;
        size_t                          namelen = strlen(session_info->username);
        mapfile = fopen(globus_i_s3_map_filename, "r");

        while (!feof(mapfile))
        {
            const char                 *l = NULL;

            l = fgets(userline, sizeof(userline), mapfile);
            if (l == NULL)
            {
                result = GlobusGFSErrorGeneric("Unknown user");
                goto fail_parse;
            }
            else if (l[strlen(l)-1] != '\n')
            {
                // line too long
                result = GlobusGFSErrorMemory("user");
                goto fail_parse;
            }
            else if (l[0] == '#')
            {
                continue;
            }
            else if (strncmp(userline, session_info->username, namelen) == 0
                    && userline[namelen] == ';')
            {
                userline[strlen(userline)-1] = 0;
                memmove(userline,
                        userline + namelen + 1,
                        strlen(userline + namelen + 1) + 1);
                break;
            }
        }
fail_parse:
        fclose(mapfile);
        if (result != GLOBUS_SUCCESS)
        {
            goto bad_username;
        }
    }
    else
    {
        if (strlen(session_info->username) > (sizeof(userline) - 1))
        {
            result = GlobusGFSErrorGeneric("Invalid username");
            goto bad_username;
        }
        strcpy(userline, session_info->username);
    }

    if ((secretAccessKey = strchr(accessKeyId, ';')) != NULL)
    {
        *secretAccessKey++ = '\0';
    }
    if (accessKeyId == NULL || secretAccessKey == NULL)
    {
        result = GlobusGFSErrorParameter("username");
        goto bad_username;
    }
    if ((securityToken = strchr(secretAccessKey, ';')) != NULL)
    {
        *securityToken++ = '\0';
    }

    s3_handle = malloc(sizeof(globus_i_s3_handle_t));
    if (s3_handle == NULL)
    {
        result = GlobusGFSErrorMemory("s3_handle");
        goto malloc_fail;
    }
    *s3_handle = (globus_i_s3_handle_t)
    {
        .protocol                     = globus_i_s3_https
                                        ? S3ProtocolHTTPS
                                        : S3ProtocolHTTP,
        .uriStyle                     = globus_i_s3_use_virtual_host
                                        ? S3UriStyleVirtualHost
                                        : S3UriStylePath,
        .accessKeyId                  = strdup(accessKeyId),
        .secretAccessKey              = strdup(secretAccessKey),
        .securityToken                = securityToken ? strdup(securityToken) : NULL
    };

    if (s3_handle->accessKeyId == NULL)
    {
        result = GlobusGFSErrorMemory("accessKeyId");
        goto strdup_fail;
    }

    if (s3_handle->secretAccessKey == NULL)
    {
        result = GlobusGFSErrorMemory("secretAccessKey");
        goto strdup_fail;
    }

    if (result != GLOBUS_SUCCESS)
    {
strdup_fail:
        free(s3_handle->secretAccessKey);
        free(s3_handle->accessKeyId);
malloc_fail:
        free(s3_handle);
        s3_handle = NULL;
    }
bad_username:
    globus_gridftp_server_finished_session_start(
            op,
            result, 
            s3_handle,
            s3_handle ? s3_handle->accessKeyId : "ERROR",
            NULL);
}
/* globus_l_s3_start() */
// vim: ft=c:
